﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using System.Xml.Linq;
using Hardcodet.LiveWriter.ApplyCssPlugin.Properties;
using OpenFileDialog=Microsoft.Win32.OpenFileDialog;


namespace Hardcodet.LiveWriter.ApplyCssPlugin
{
  public static class ContentParser
  {
    /// <summary>
    /// Pattern that extracts class names from typical CSS files.
    /// Not too robust, but will do for now.
    /// </summary>
    private const string CssClassPattern =
        @"\.[-]?[_a-zA-Z][_a-zA-Z0-9-]*|[^\0-\177]*\\[0-9a-f]{1,6}(\r\n[ \n\r\t\f])?|\\[^\n\r\f0-9a-f]*";

    private const string CommentsPattern =
        @"(?<commentblock>((?m:^[\t ]*\/{2}[^\n\r\v\f]+[\n\r\v\f]*){2,})|(\/\*[\w\W]*?\*\/))";


    /// <summary>
    /// Displays a file dialog and returns the parsed files.
    /// </summary>
    /// <returns>A list of parsed class names, or null, if the user cancelled the
    /// operation.</returns>
    public static bool TryGetClassNames(out List<string> classNames)
    {
      var dlg = new OpenFileDialog();
      dlg.DefaultExt = ".css"; // Default file extension
      dlg.Filter = Resources.CssFileFilter; // Filter files by extension

      bool? result = dlg.ShowDialog();
      
      if (result == true)
      {
        classNames = GetClassNames(dlg.FileName);
        return true;
      }

      classNames = null;
      return false;
    }


    /// <summary>
    /// Parses a given CSS files and extracts all class names.
    /// </summary>
    /// <param name="cssFile">The file to be parsed.</param>
    /// <returns>A list of parsed class names.</returns>
    public static List<string> GetClassNames(string cssFile)
    {
      string css = File.ReadAllText(cssFile);

      //remove comments
      css = Regex.Replace(css, CommentsPattern, "");

      //get classes
      var matches = Regex.Matches(css, CssClassPattern, RegexOptions.IgnoreCase | RegexOptions.Multiline);

      //extract class names
      List<string> classNames = new List<string>();
      foreach (Match match in matches)
      {
        if (match.Captures.Count == 0) continue;
        string className = match.Captures[0].Value;

        if (String.IsNullOrEmpty(className)) continue;
        
        //remove the leading dot
        if (className.StartsWith("."))
        {
          className = className.Substring(1, className.Length - 1);
        }

        classNames.Add(className);
      }

      return classNames;
    }


    public static bool InjectClassAttribute(ref string content, string className)
    {
      //get clean XHTML content, first (this might mess up the markup if the
      //user selected half a paragraph including the end or start tag of the
      //paragraph
      var doc = new HtmlAgilityPack.HtmlDocument();
      doc.LoadHtml(content);
      StringWriter w = new StringWriter();
      doc.Save(w);
      content = w.ToString();


      XElement element;
      try
      {
        element = XElement.Parse(content);
      }
      catch
      {
        //put markup that contains block elements into div tags, inline
        //into spans
        //-> if junk of text was selected that does only contain an opening
        //or closing tag, we'll end up with malformed XHTML
        bool containsBlockElements = ContainsBlockElements(content);
        string tag = containsBlockElements ? "div" : "span";

        string xml = String.Format("<{0}>{1}</{0}>", tag, content);
        try
        {
          element = XElement.Parse(xml);
        }
        catch (Exception e)
        {
          string msg = "Could not parse content {0}: {1}";
          MessageBox.Show(String.Format(msg, content, e.Message));
          return false;
        }
      }

      //set the class attribute and return the updated markup
      element.SetAttributeValue("class", className);
      content = element.ToString();
      return true;
    }



    /// <summary>
    /// Checks whether the submitted markup contains block elements or not.
    /// </summary>
    /// <param name="content"></param>
    /// <returns></returns>
    private static bool ContainsBlockElements(string content)
    {
      content = content.Replace(" ", "");
      return content.Contains("<div")
             || content.Contains("<p")
             || content.Contains("<pre")
             || content.Contains("<center ")
             || content.Contains("<blockquote ")
             || content.Contains("<hr ")
             || content.Contains("<table")
             || content.Contains("<h1")
             || content.Contains("<h2")
             || content.Contains("<h3")
             || content.Contains("<h4")
             || content.Contains("<h5")
             || content.Contains("<h6");
    }
  }
}