﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using WindowsLive.Writer.Api;

namespace Hardcodet.LiveWriter.ApplyCssPlugin
{
  /// <summary>
  /// Encapsulates the plug-in's settings.
  /// </summary>
  public class PluginSettings : INotifyPropertyChanged
  {
    /// <summary>
    /// WLW property collection.
    /// </summary>
    public IProperties PluginProperties { get; private set; }

    /// <summary>
    /// Gets the most recently used class names. The list
    /// provides a maximum of 5 entries.
    /// </summary>
    public ObservableCollection<string> MruList { get; private set; }


    private ObservableCollection<string> classNames;

    /// <summary>
    /// Gets the currently configured class names as a list.
    /// </summary>
    public ObservableCollection<string> ClassNames
    {
      get { return classNames; }
      private set
      {
        if (value == null) value = new ObservableCollection<string>();
        classNames = value;

        UpdateSettings();
        OnPropertyChanged("ClassNames");
      }
    }


    public PluginSettings(IProperties properties)
    {
      PluginProperties = properties;
      var classes = PluginProperties.GetString("ClassNames", "")
          .Split(new[] {','}, StringSplitOptions.RemoveEmptyEntries);
      classNames = new ObservableCollection<string>(classes);

      var mruItems = PluginProperties.GetString("MruList", "")
          .Split(new[] {','}, StringSplitOptions.RemoveEmptyEntries);
      MruList = new ObservableCollection<string>(mruItems);


      //clean up invalid MRU settings
      var invalidItems = mruItems.Where(name => !classNames.Contains(name));
      if (invalidItems.Count() > 0)
      {
        foreach (var item in invalidItems)
        {
          MruList.Remove(item);
        }
      }
    }


    /// <summary>
    /// Adds a new class to the settings.
    /// </summary>
    public void AddClass(string className)
    {
      StringComparer comparer = StringComparer.InvariantCultureIgnoreCase;
      if (classNames.Contains(className, comparer))
      {
        return;
      }

      ClassNames.Add(className);
      ClassNames = new ObservableCollection<string>(classNames.OrderBy(s => s, comparer));      
      UpdateSettings();
      
    }


    /// <summary>
    /// Removes a given class.
    /// </summary>
    public void RemoveClass(string className)
    {
      MruList.Remove(className);
      classNames.Remove(className);
      UpdateSettings();
    }


    /// <summary>
    /// Updates the underlying WLW settings.
    /// </summary>
    private void UpdateSettings()
    {
      StringBuilder builder = new StringBuilder();
      foreach (var name in classNames)
      {
        builder.Append(name).Append(",");
      }
      PluginProperties.SetString("ClassNames", builder.ToString());


      builder = new StringBuilder();
      foreach (var mruItem in MruList)
      {
        builder.Append(mruItem).Append(",");
      }
      PluginProperties.SetString("MruList", builder.ToString());
    }


    /// <summary>
    /// Updates the list of most recently used class names.
    /// </summary>
    /// <param name="className">The most recently used class name.</param>
    public void UpdateMruList(string className)
    {
      if (className != null)
      {
        MruList.Remove(className);
        MruList.Insert(0, className);
      }

      //shape list
      while (MruList.Count > 5)
      {
        MruList.RemoveAt(5);
      }

      UpdateSettings();
    }


    #region INotifyPropertyChanged event

    ///<summary>
    ///Occurs when a property value changes.
    ///</summary>
    public event PropertyChangedEventHandler PropertyChanged;


    /// <summary>
    /// Raises the <see cref="PropertyChanged"/> event for
    /// a given property.
    /// </summary>
    /// <param name="propertyName">The name of the changed property.</param>
    protected void OnPropertyChanged(string propertyName)
    {
      if (PropertyChanged != null)
      {
        PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
      }
    }


    #endregion
  }
}